from typing import Any, Generator, cast, Type
from pydantic import create_model


class PydanticCompatible:
    """
    See the [link](https://github.com/crimson206/types/blob/main/example/addon/pydantic_compatible.ipynb)
    """

    validate_base_index: int = 0

    @classmethod
    def __get_validators__(cls) -> Generator:
        yield cls.validate

    @classmethod
    def validate(cls, value: Any, _):
        try:
            DynamicTester = create_model(
                "DynamicTester", base=(cls.__bases__[cls.validate_base_index], ...)
            )
        except AttributeError as e:
            e
            raise Exception(
                "The innertype of your custom type class must be placed as the first base of the type class as default. If you want to place it another position, specify the position using `validate_base_index`. See the link, https://github.com/crimson206/types/blob/main/example/addon/pydantic_compatible.ipynb"
            )

        DynamicTester(base=value)
        return cast(Type[cls], value)  # type: ignore



class AnnotatedType:
    """
    Enables metadata injection using square brackets ([]) on class types.
    Inherit this class to use metadata annotations with `[]`.
    """
    def __class_getitem__(cls, params):
        cls.__metadata__ = params
        return cls


def annotated_type(cls):
    """
    Adds metadata injection capability to a class via square brackets ([]).
    Use this decorator to enable metadata annotations on classes.
    """
    def class_getitem(cls, params):
        cls.__metadata__ = params
        return cls

    cls.__class_getitem__ = classmethod(class_getitem)
    return cls



from crimson.types.addon.intelli_type import IntelliHolder, T
from crimson.types.addon.pydantic import PydanticCompatible
from crimson.types.addon.annotated import annotated_type


@annotated_type
class TypesPack(PydanticCompatible, IntelliHolder[T]):
    """
    If you want to use the __metadata__, this must be the first base of your custom type.

    Go to docs(examples): [Link](https://github.com/crimson206/types/blob/main/example/addon/packs.ipynb)
    """

    # validate_base_index = 1
